/*
------------------------------------------------------------
	buildst.c - Build headers and data from name list
------------------------------------------------------------
 *  David Olofson, 2001
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

char prefix[80] = "SYM_";

int readline(FILE *in, char *buf)
{
	while(1)
	{
		char ch = getc(in);
		if(ch == EOF)
		{
			*buf = 0;
			return -1;
		}

		switch(ch)
		{
		  case '\t':
			break;

		  default:
			if(ch < ' ')
			{
				*buf = 0;
				return 0;
			}
		}
		*buf++ = ch;
	}
}


int generate_data(FILE *in, FILE *out, const char *name)
{
	char buf[80];
	fprintf(out, "const char %s_data[] = {\n", name);
//	fprintf(out, "\t\"<unknown>\\0\"\n");
	while(readline(in, buf) == 0)
	{
		int offs = 0;
		if(buf[0] == '!')
			continue;
		if(buf[0] == '@')
			continue;
		if(buf[0] == '#')
			for(offs = 2; buf[offs-1] != '#'; ++offs)
				;
		fprintf(out, "\t\"%s\\0\"\n", buf + offs);
	}
	fprintf(out,	"\t\"\\0\"\n"
			"};\n");
	return 0;
}


int generate_symbols(FILE *in, FILE *out, const char *name)
{
	int ind = 0;
	char buf[256];
	fprintf(out, "typedef enum\n{\n");
	while(readline(in, buf) == 0)
	{
		int i;
		int offs = 0;
		if(buf[0] == '!')
			continue;
		if(buf[0] == '@')
		{
			strcpy(prefix, buf + 1);
			continue;
		}
		if(buf[0] == '#')
		{
			for(offs = 1; buf[offs] != '#'; ++offs)
				;
			buf[offs] = 0;
			offs = 1;
		}
		for(i = strlen(buf + offs)-1; i >= 0; --i)
			buf[i + offs] = toupper(buf[i + offs]);
		fprintf(out, "\t%s%s = %d,\n", prefix, buf + offs, ind);
		++ind;
	}
	fprintf(out, "} %s_t;\n"
			"\n"
			"extern const char %s_data[];\n", name, name);
	return 0;
}


int main(int argc, char *argv[])
{
	if((argc == 3) && (argv[1][0] == 'd'))
		return generate_data(stdin, stdout, argv[2]);
	if((argc == 3) && (argv[1][0] == 's'))
		return generate_symbols(stdin, stdout, argv[2]);
	else
	{
		fprintf(stderr, "usage: %s [ds] <name>\n"
				"     Use 'd' to generate data string\n"
				"     Use 's' to generate symbol enumeration\n"
				"     <name> is the name prefix for the data\n"
				"            and symbol definitions.\n",
				argv[0]);
		return 1;
	}

	return 0;
}
